/* eslint-disable @typescript-eslint/no-empty-function */
/* eslint-disable @typescript-eslint/no-unused-vars */
// eslint-disable-next-line max-classes-per-file
import focusFirst from "./helpers/focusFirst";
import nestedProperty from "./helpers/nestedProperty";
import observer from "./helpers/observer";
import onEvent from "./helpers/onEvent";
import { WUPcssHidden } from "./styles";
// theoritcally such single appending is faster than using :host inside shadowComponent
const appendedStyles = new Set();
let lastUniqueNum = 0;
const allObservedOptions = new WeakMap();
/** Basic abstract class for every component in web-ui-pack */
export default class WUPBaseElement extends HTMLElement {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    /** Reference to global style element used by web-ui-pack */
    static get $refStyle() {
        return window.WUPrefStyle || null;
    }
    static set $refStyle(v) {
        window.WUPrefStyle = v || undefined;
    }
    /** StyleContent related to component */
    static get $style() {
        return "";
    }
    /** StyleContent related to component & inherrited components */
    static get $styleRoot() {
        return `:root {
          --base-bg: #fff;
          --base-text: #232323;
          --base-focus: #00778d;
          --base-btn-bg: #009fbc;
          --base-btn-text: #fff;
          --base-btn-focus: #005766;
          --base-btn2-bg: var(--base-btn-text);
          --base-btn2-text: var(--base-btn-bg);
          --base-btn3-bg: var(--base-bg);
          --base-btn3-text: inherit;
          --base-sep: #e4e4e4;
          --border-radius: 6px;
          --anim-time: 200ms;
          --anim: var(--anim-time) cubic-bezier(0, 0, 0.2, 1) 0ms;
        }`;
    }
    /** Get unique id for html elements; Every getter returns new id */
    static get $uniqueId() {
        return `wup${++lastUniqueNum}`;
    }
    static get classNameHidden() {
        return "wup-hidden";
    }
    _opts = {};
    constructor() {
        super();
        if (!this.#ctr.$refStyle) {
            this.#ctr.$refStyle = document.createElement("style");
            /* from https://snook.ca/archives/html_and_css/hiding-content-for-accessibility  */
            this.#ctr.$refStyle.append(`.${this.#ctr.classNameHidden}, [${this.#ctr.classNameHidden}] {${WUPcssHidden}}`);
            document.head.prepend(this.#ctr.$refStyle);
        }
        const refStyle = this.#ctr.$refStyle;
        // setup options to be observable
        setTimeout(() => {
            // cast options to observed
            const prev = this.$options;
            // get from cache
            let o = allObservedOptions.get(this.#ctr);
            if (o === undefined) {
                // @ts-ignore
                const arr = this.#ctr.observedOptions;
                o = arr?.length ? new Set(arr) : null;
                allObservedOptions.set(this.#ctr, o);
            }
            Object.defineProperty(this, "$options", {
                set: this.#setOptions,
                get: () => {
                    const watched = o;
                    if (!watched?.size) {
                        return this._opts;
                    }
                    // cast to observed only if option was retrieved: to optimize init-performance
                    if (!this.#optsObserved) {
                        this.#optsObserved = observer.make(this._opts);
                        this.#removeObserved = observer.onChanged(this.#optsObserved, (e) => {
                            this.#isReady && e.props.some((p) => watched.has(p)) && this.gotOptionsChanged(e);
                        });
                    }
                    return this.#optsObserved;
                },
            });
            this._opts = prev;
            this.#setOptions(prev);
        });
        // setup styles
        if (!appendedStyles.has(this.tagName)) {
            appendedStyles.add(this.tagName);
            // autoBind functions (recursive until HMTLElement)
            const findAllProtos = (t, protos) => {
                const p = Object.getPrototypeOf(t);
                if (p !== HTMLElement.prototype) {
                    protos.push(p.constructor);
                    // Object.getOwnPropertyNames(p).forEach((s) => {
                    //   const k = s as keyof Omit<WUPBaseElement, keyof HTMLElement | "$isReady">;
                    //   const desc = Object.getOwnPropertyDescriptor(p, s) as PropertyDescriptor;
                    //   if (typeof desc.value === "function" && s !== "constructor") {
                    //     this[k] = (this[k] as Function).bind(this);
                    //   }
                    // });
                    findAllProtos(p, protos);
                }
                return protos;
            };
            const protos = findAllProtos(this, []);
            protos.reverse().forEach((p) => {
                // append $styleRoot
                if (!appendedStyles.has(p.name)) {
                    appendedStyles.add(p.name);
                    if (Object.prototype.hasOwnProperty.call(p, "$styleRoot")) {
                        const s = p.$styleRoot;
                        s && refStyle.append(s);
                    }
                }
            });
            const c = protos[protos.length - 1].$style;
            refStyle.append(c.replace(/:host/g, `${this.tagName}`));
        }
    }
    /* rawOptions ($options is observed) */
    #optsObserved;
    #removeObserved;
    #setOptions(v) {
        if (!v) {
            throw new Error("$options can't be empty");
        }
        const prev = this._opts;
        this._opts = v;
        // unsubscribe from previous events here
        this.#removeObserved?.call(this);
        this.#optsObserved = undefined;
        this.#removeObserved = undefined;
        const observedOptions = allObservedOptions.get(this.#ctr);
        if (!observedOptions?.size) {
            return;
        }
        if (this.#isReady && prev.valueOf() !== v.valueOf()) {
            const props = [];
            // eslint-disable-next-line no-restricted-syntax
            for (const [k] of observedOptions.entries()) {
                if (this._opts[k] !== prev[k]) {
                    props.push(k);
                }
            }
            props.length && this.gotOptionsChanged({ props, target: this._opts });
        }
    }
    /** Returns true if element is appended (result of setTimeout on connectedCallback) */
    get $isReady() {
        return this.#isReady;
    }
    /** Try to focus self or first possible children; returns true if succesful */
    focus() {
        return focusFirst(this);
    }
    /** Remove focus from element on any nested active element */
    blur() {
        const ae = document.activeElement;
        if (ae === this) {
            super.blur();
        }
        else if (ae instanceof HTMLElement && this.includes(ae)) {
            ae.blur();
        }
    }
    #isReady = false;
    /** Called when element is added to document */
    gotReady() {
        this.#readyTimeout = undefined;
        this.#isReady = true;
        this._isStopChanges = true;
        this.gotChanges(null);
        this._isStopChanges = false;
        this.#readyTimeout = setTimeout(() => {
            (this.autofocus || this._opts.autoFocus) && this.focus();
            this.#readyTimeout = undefined;
        }); // timeout to wait for options
    }
    /** Called when element is removed from document */
    gotRemoved() {
        this.#isReady = false;
        this.dispose();
    }
    /** Called on Init and every time as options/attributes changed */
    gotChanges(propsChanged) { }
    /** Called when element isReady and at least one of observedOptions is changed */
    gotOptionsChanged(e) {
        this._isStopChanges = true;
        e.props.forEach((p) => this.removeAttribute(p)); // remove related attributes otherwise impossible to override
        this.gotChanges(e.props);
        this._isStopChanges = false;
    }
    /** Called once on Init */
    gotRender() { }
    #isFirstConn = true;
    #readyTimeout;
    /** Browser calls this method when the element is added to the document */
    connectedCallback() {
        // async requires otherwise attributeChangedCallback doesn't set immediately
        this.#readyTimeout = setTimeout(() => this.gotReady.call(this));
        if (this.#isFirstConn) {
            this.#isFirstConn = false;
            this.gotRender();
        }
    }
    /** Browser calls this method when the element is removed from the document;
     * (can be called many times if an element is repeatedly added/removed) */
    disconnectedCallback() {
        this.#readyTimeout && clearTimeout(this.#readyTimeout);
        this.#readyTimeout = undefined;
        this.gotRemoved();
    }
    _isStopChanges = true;
    #attrTimer;
    #attrChanged;
    /** Called when element isReady and one of observedAttributes is changed */
    gotAttributeChanged(name, oldValue, newValue) {
        // debounce filter
        if (this.#attrTimer) {
            this.#attrChanged.push(name);
            return;
        }
        this.#attrChanged = [name];
        this.#attrTimer = setTimeout(() => {
            this.#attrTimer = undefined;
            this._isStopChanges = true;
            const keys = Object.keys(this._opts);
            const keysNormalized = []; // cache to boost performance via exlcuding extra-lowerCase
            this.#attrChanged.forEach((a) => {
                keys.some((k, i) => {
                    let kn = keysNormalized[i];
                    if (!kn) {
                        kn = k.toLowerCase();
                        keysNormalized.push(kn);
                    }
                    if (kn === a) {
                        delete this._opts[k];
                        return true;
                    }
                    return false;
                });
            }); // otherwise attr can't override option if attribute removed
            this.gotChanges(this.#attrChanged);
            this._isStopChanges = false;
            this.#attrChanged = undefined;
        });
    }
    /** Browser calls this method when attrs pointed in observedAttributes is changed */
    attributeChangedCallback(name, oldValue, newValue) {
        this.#isReady &&
            !this._isStopChanges &&
            oldValue !== newValue &&
            this.gotAttributeChanged(name, oldValue, newValue);
    }
    dispatchEvent(ev, eventInit) {
        if (typeof ev === "string")
            ev = new Event(ev, eventInit);
        return super.dispatchEvent(ev);
    }
    addEventListener(type, listener, options) {
        return super.addEventListener(type, listener, options);
    }
    removeEventListener(type, listener, options) {
        return super.removeEventListener(type, listener, options);
    }
    /* eslint-enable max-len */
    /** Calls dispatchEvent and returns created event */
    fireEvent(type, eventInit) {
        const ev = new Event(type, eventInit);
        super.dispatchEvent(ev);
        return ev;
    }
    /** Array of removeEventListener() that called on remove */
    disposeLst = [];
    /** Add event listener and remove after component removed; @options.passive=true by default */
    appendEvent(...args) {
        // self-removing when option.once
        if (args[3]?.once) {
            const listener = args[2];
            args[2] = function wrapper(...args2) {
                const v = listener.call(this, ...args2);
                remove();
                return v;
            };
        }
        // @ts-expect-error
        const r = onEvent(...args);
        this.disposeLst.push(r);
        const remove = () => {
            const i = this.disposeLst.indexOf(r);
            if (i !== -1) {
                r();
                this.disposeLst.splice(i, 1);
            }
        };
        return remove;
    }
    /** Remove events/functions that was appended */
    dispose() {
        this.disposeLst.forEach((f) => f()); // remove possible previous event listeners
        this.disposeLst.length = 0;
    }
    /** Returns true if el is instance of Node and contains pointer element */
    includes(el) {
        return el instanceof Node && this.contains(el);
    }
    getBoolAttr(attr, alt) {
        const a = this.getAttribute(attr);
        return a === null ? alt : a !== "false";
    }
    /** Parse attribute and return result; if attr missed or invalid => returns this._opts[attr] */
    getNumAttr(attr) {
        const a = this.getAttribute(attr);
        if (a == null || a === "") {
            return this._opts[attr];
        }
        const v = +a;
        if (Number.isNaN(v)) {
            console.error(`${this.tagName}. Expected number for attribute [${attr}] but pointed '${a}'`);
        }
        return v;
    }
    /** Returns value from window[key] according to [attr]="key" or $options[key] if attr is missed */
    getRefAttr(attr) {
        const a = this.getAttribute(attr);
        if (a == null || a === "") {
            return this._opts[attr];
        }
        const v = nestedProperty.get(window, a);
        if (v === undefined) {
            console.error(`${this.tagName}. Value not found according to attribute [${attr}] in '${a.startsWith("window.") ? a : `window.${a}`}'`);
        }
        return v;
    }
    /**
     * Remove attr if value falseOrEmpty; set '' or 'true' if true for HTMLELement
     * @param isSetEmpty set if need '' instead of 'value'
     */
    setAttr(attr, v, isSetEmpty) {
        v ? this.setAttribute(attr, isSetEmpty ? "" : v) : this.removeAttribute(attr);
    }
}
// testcase: check if gotChanges sensitive to attr-case
