import WUPBaseElement from "./baseElement";
declare const tagName = "wup-circle";
declare global {
    namespace WUP.Circle {
        interface Defaults {
            /** Width of each segment; expected 1..100 (perecentage)
             * @defaultValue 10 */
            width: number;
            /** Border/corner radius of each segment; expected 0..0.5 where 0.5 == 50% of `$options.wdith`
             * @defaultValue 0.25 */
            corner: number;
            /** Enable background circle
             * @defaultValue true */
            back: boolean;
            /** Angle from that rendering is started -360..360 (degrees)
             * @defaultValue 0 */
            from: number;
            /** Angle from that rendering is started -360..360 (degrees)
             * @defaultValue 360 */
            to: number;
            /** Min possible value that fits `options.from`
             * @defaultValue 0 */
            min: number;
            /** Max possible value that fits `options.to`
             * @defaultValue 100 */
            max: number;
            /** Space between segments; expected 0...20 (degrees)
             * @defaultValue 2 */
            space: number;
        }
        interface Options extends Defaults {
            /** Items related to circle-segments */
            items: Array<{
                value: number;
                color?: string;
            }>;
        }
        interface Attributes extends WUP.Base.toJSX<Partial<Pick<Options, "back" | "width" | "from" | "to" | "items" | "corner" | "min" | "max" | "space">>> {
        }
        interface JSXProps<C = WUPCircleElement> extends WUP.Base.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPCircleElement;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Circle.JSXProps;
        }
    }
}
/** Arc/circle chart based on SVG
 * @example
 * <wup-circle
 *   back="true"
 *   from="0"
 *   to="360"
 *   space="2"
 *   min="0"
 *   max="100"
 *   width="14"
 *   corner="0.25"
 *   items="window.circleItems"
 *  ></wup-circle>
 * // or JS/TS
 * const el = document.createElement("wup-circle");
 * el.$options.items = [{value:20}]; // etc.
 * document.body.appendChild(el); */
export default class WUPCircleElement extends WUPBaseElement {
    #private;
    static get observedOptions(): Array<keyof WUP.Circle.Options>;
    static get observedAttributes(): Array<LowerKeys<WUP.Circle.Attributes>>;
    static get $styleRoot(): string;
    static get $style(): string;
    static $defaults: WUP.Circle.Defaults;
    $options: WUP.Circle.Options;
    $refSVG: SVGSVGElement | SVGCircleElement | SVGClipPathElement | SVGDefsElement | SVGDescElement | SVGEllipseElement | SVGFEBlendElement | SVGFEColorMatrixElement | SVGFEComponentTransferElement | SVGFECompositeElement | SVGFEConvolveMatrixElement | SVGFEDiffuseLightingElement | SVGFEDisplacementMapElement | SVGFEDistantLightElement | SVGFEDropShadowElement | SVGFEFloodElement | SVGFEFuncAElement | SVGFEFuncBElement | SVGFEFuncGElement | SVGFEFuncRElement | SVGFEGaussianBlurElement | SVGFEImageElement | SVGFEMergeElement | SVGFEMergeNodeElement | SVGFEMorphologyElement | SVGFEOffsetElement | SVGFEPointLightElement | SVGFESpecularLightingElement | SVGFESpotLightElement | SVGFETileElement | SVGFETurbulenceElement | SVGFilterElement | SVGForeignObjectElement | SVGGElement | SVGImageElement | SVGLineElement | SVGLinearGradientElement | SVGMarkerElement | SVGMaskElement | SVGMetadataElement | SVGPathElement | SVGPatternElement | SVGPolygonElement | SVGPolylineElement | SVGRadialGradientElement | SVGRectElement | SVGStopElement | SVGSwitchElement | SVGSymbolElement | SVGTextElement | SVGTextPathElement | SVGTSpanElement | SVGUseElement | SVGViewElement | SVGStyleElement | SVGAElement | SVGScriptElement | SVGTitleElement | SVGAnimateElement | SVGAnimateMotionElement | SVGAnimateTransformElement | SVGMPathElement | SVGSetElement;
    $refItems: SVGSVGElement | SVGCircleElement | SVGClipPathElement | SVGDefsElement | SVGDescElement | SVGEllipseElement | SVGFEBlendElement | SVGFEColorMatrixElement | SVGFEComponentTransferElement | SVGFECompositeElement | SVGFEConvolveMatrixElement | SVGFEDiffuseLightingElement | SVGFEDisplacementMapElement | SVGFEDistantLightElement | SVGFEDropShadowElement | SVGFEFloodElement | SVGFEFuncAElement | SVGFEFuncBElement | SVGFEFuncGElement | SVGFEFuncRElement | SVGFEGaussianBlurElement | SVGFEImageElement | SVGFEMergeElement | SVGFEMergeNodeElement | SVGFEMorphologyElement | SVGFEOffsetElement | SVGFEPointLightElement | SVGFESpecularLightingElement | SVGFESpotLightElement | SVGFETileElement | SVGFETurbulenceElement | SVGFilterElement | SVGForeignObjectElement | SVGGElement | SVGImageElement | SVGLineElement | SVGLinearGradientElement | SVGMarkerElement | SVGMaskElement | SVGMetadataElement | SVGPathElement | SVGPatternElement | SVGPolygonElement | SVGPolylineElement | SVGRadialGradientElement | SVGRectElement | SVGStopElement | SVGSwitchElement | SVGSymbolElement | SVGTextElement | SVGTextPathElement | SVGTSpanElement | SVGUseElement | SVGViewElement | SVGStyleElement | SVGAElement | SVGScriptElement | SVGTitleElement | SVGAnimateElement | SVGAnimateMotionElement | SVGAnimateTransformElement | SVGMPathElement | SVGSetElement;
    $refLabel?: HTMLElement;
    protected _opts: WUP.Circle.Options;
    /** Creates new svg-part */
    protected make<K extends keyof SVGElementTagNameMap>(tag: string): SVGElementTagNameMap[K];
    protected gotChanges(propsChanged: Array<keyof WUP.Circle.Options> | null): void;
    _animation?: WUP.PromiseCancel<boolean>;
    protected gotRenderItems(): void;
    /** Called on every changeEvent */
    protected gotRender(): void;
    /** Called every time as need text-value */
    renderLabel(label: HTMLElement, percent: number, rawValue: number): void;
    /** Returns svg-path for Arc according to options */
    protected drawArc(angleFrom: number, angleTo: number): string;
}
/** Returns svg-path for Cirle according to options */
export declare function drawCircle(center: [number, number], r: number, width: number): string;
/** Returns svg-path for Arc according to options */
export declare function drawArc(center: [number, number], r: number, width: number, angleFrom: number, angleTo: number, cornerR: number): string;
export {};
