import WUPPopupElement from "../popup/popupElement";
import WUPTextControl from "./text";
export declare const enum ShowCases {
    /** When $showMenu() called programmatically; Don't use it for $options (it's for nested cycle) */
    onManualCall = 1,
    /** When control got focus */
    onFocus = 2,
    /** When user clicks on control (beside editable not-empty input) */
    onClick = 4,
    /** When user clicks on input (by default it's disabled rule to allow user to work with input without popup hide/show blinks) */
    onClickInput = 16,
    /** When user types text */
    onInput = 32,
    /** When user presses arrowUp/arrowDown */
    onPressArrowKey = 64
}
export declare const enum HideCases {
    onManualCall = 0,
    onClick = 1,
    onSelect = 2,
    onFocusLost = 3,
    OnPressEsc = 4,
    OnPressEnter = 5,
    OnOptionsChange = 6
}
declare global {
    namespace WUP.BaseCombo {
        interface EventMap extends WUP.Text.EventMap {
            /** Fires after popup-menu is shown (after animation finishes) */
            $showMenu: Event;
            /** Fires after popup is hidden (after animation finishes) */
            $hideMenu: Event;
        }
        interface ValidityMap extends Omit<WUP.Text.ValidityMap, "min" | "max" | "email"> {
        }
        interface Defaults<T = string, VM = ValidityMap> extends WUP.Text.Defaults<T, VM> {
            /** Case when menu-popup to show; WARN ShowCases.inputClick doesn't work without ShowCases.click
             * @defaultValue onPressArrowKey | onClick | onFocus */
            showCase: ShowCases;
        }
        interface Options<T = string, VM = ValidityMap> extends WUP.Text.Options<T, VM>, Defaults<T, VM> {
            /** Set true to make input not editable but allow to user select items via popup-menu (ordinary dropdown mode) */
            readOnlyInput?: boolean;
        }
        interface Attributes extends WUP.Text.Attributes {
        }
        interface JSXProps<C = WUPBaseComboControl> extends WUP.Text.JSXProps<C>, Attributes {
        }
    }
}
/** Base abstract form-control for any control with popup-menu (Dropdown, Datepicker etc.) */
export default abstract class WUPBaseComboControl<ValueType = any, EventMap extends WUP.BaseCombo.EventMap = WUP.BaseCombo.EventMap> extends WUPTextControl<ValueType, EventMap> {
    #private;
    static get observedOptions(): Array<string>;
    static get $style(): string;
    static $defaults: WUP.BaseCombo.Defaults<any>;
    $options: WUP.BaseCombo.Options;
    protected _opts: WUP.BaseCombo.Options<string, WUP.BaseCombo.ValidityMap>;
    /** Returns if popup-menu is opened */
    get $isOpen(): boolean;
    /** Hide popup-menu
     * @returns Promise resolved by animation time */
    $hideMenu(): Promise<void>;
    /** Show popup-menu
     * @returns Promise resolved resolved by animation time */
    $showMenu(): Promise<void>;
    /** Reference to popupMenu */
    $refPopup?: WUPPopupElement;
    protected renderControl(): void;
    protected gotChanges(propsChanged: Array<keyof WUP.BaseCombo.Options> | null): void;
    gotFormChanges(propsChanged: Array<keyof WUP.Form.Options | keyof WUP.BaseCombo.Options> | null): void;
    /** Called when need to create menu in opened popup */
    protected abstract renderMenu(popup: WUPPopupElement, menuId: string): Promise<HTMLElement> | HTMLElement;
    /** Called when need to transfer current value to input */
    protected abstract valueToInput(v: ValueType | undefined): string | Promise<string>;
    /** Override to change show-behavior */
    canShowMenu(showCase: ShowCases, e?: MouseEvent | FocusEvent | null): boolean;
    protected goShowMenu(showCase: ShowCases, e?: MouseEvent | FocusEvent | null, isNeedWait?: boolean): Promise<WUPPopupElement | null>;
    /** Override to change hide-behavior */
    canHideMenu(hideCase: HideCases, e?: MouseEvent | FocusEvent | null): boolean;
    protected _isHidding?: true;
    protected goHideMenu(hideCase: HideCases, e?: MouseEvent | FocusEvent | null): Promise<boolean>;
    _focusedMenuItem?: HTMLElement | null;
    /** Focus/resetFocus for item (via aria-activedescendant) */
    protected focusMenuItem(next: HTMLElement | null): void;
    _selectedMenuItem?: HTMLElement | null;
    /** Select item (set aria-selected and scroll to) */
    protected selectMenuItem(next: HTMLElement | null): void;
    protected gotKeyDown(e: KeyboardEvent): Promise<void>;
    protected gotInput(e: WUP.Text.GotInputEvent, allowSuper?: boolean): void;
    protected gotFocusLost(): void;
    /** Called when user selected new value from menu and need to hide menu */
    protected selectValue(v: ValueType): void;
    protected resetInputValue(): void;
    protected setInputValue(v: ValueType | undefined): void;
    /** Called when popup must be removed (by focus out OR if control removed) */
    protected removePopup(): void;
    protected gotRemoved(): void;
}
