import WUPPopupElement from "../popup/popupElement";
import popupListen from "../popup/popupListen";
import WUPBaseControl from "./baseControl";
import WUPTextControl from "./text";
export var ShowCases;
(function (ShowCases) {
    /** When $showMenu() called programmatically; Don't use it for $options (it's for nested cycle) */
    ShowCases[ShowCases["onManualCall"] = 1] = "onManualCall";
    /** When control got focus */
    ShowCases[ShowCases["onFocus"] = 2] = "onFocus";
    /** When user clicks on control (beside editable not-empty input) */
    ShowCases[ShowCases["onClick"] = 4] = "onClick";
    /** When user clicks on input (by default it's disabled rule to allow user to work with input without popup hide/show blinks) */
    ShowCases[ShowCases["onClickInput"] = 16] = "onClickInput";
    /** When user types text */
    ShowCases[ShowCases["onInput"] = 32] = "onInput";
    /** When user presses arrowUp/arrowDown */
    ShowCases[ShowCases["onPressArrowKey"] = 64] = "onPressArrowKey";
})(ShowCases || (ShowCases = {}));
export var HideCases;
(function (HideCases) {
    HideCases[HideCases["onManualCall"] = 0] = "onManualCall";
    HideCases[HideCases["onClick"] = 1] = "onClick";
    HideCases[HideCases["onSelect"] = 2] = "onSelect";
    HideCases[HideCases["onFocusLost"] = 3] = "onFocusLost";
    HideCases[HideCases["OnPressEsc"] = 4] = "OnPressEsc";
    HideCases[HideCases["OnPressEnter"] = 5] = "OnPressEnter";
    HideCases[HideCases["OnOptionsChange"] = 6] = "OnOptionsChange";
})(HideCases || (HideCases = {}));
/** Base abstract form-control for any control with popup-menu (Dropdown, Datepicker etc.) */
export default class WUPBaseComboControl extends WUPTextControl {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    static get observedOptions() {
        const arr = super.observedOptions;
        arr.push("readOnlyInput");
        return arr;
    }
    static get $style() {
        return `${super.$style}
      :host {
        cursor: pointer;
      }
      :host input {
        cursor: text;
      }
      :host input:placeholder-shown,
      :host input:read-only {
        cursor: pointer;
      }
      :host label::after {
        content: "";
        -webkit-mask-image: var(--ctrl-icon-img);
        mask-image: var(--ctrl-icon-img);
      }
      @media not all and (prefers-reduced-motion) {
        :host label::after {
          transition: transform var(--anim);
        }
      }
      :host button[clear] {
        margin: 0;
      }
      :host [menu] {
        padding: 0;
        max-height: 300px;
        z-index: 90010;
      }`;
    }
    static $defaults = {
        ...WUPTextControl.$defaults,
        validationRules: {
            ...WUPBaseControl.$defaults.validationRules,
            _mask: WUPTextControl.$defaults.validationRules._mask,
            _parse: WUPTextControl.$defaults.validationRules._parse,
        },
        showCase: 4 /* ShowCases.onClick */ | 2 /* ShowCases.onFocus */ | 64 /* ShowCases.onPressArrowKey */,
    };
    // @ts-expect-error reason: validationRules is different
    $options = {
        ...this.#ctr.$defaults,
    };
    // @ts-expect-error reason: validationRules is different
    _opts = this.$options;
    #isOpen = false;
    /** Returns if popup-menu is opened */
    get $isOpen() {
        return this.#isOpen;
    }
    /** Hide popup-menu
     * @returns Promise resolved by animation time */
    async $hideMenu() {
        await this.goHideMenu(0 /* HideCases.onManualCall */);
    }
    /** Show popup-menu
     * @returns Promise resolved resolved by animation time */
    async $showMenu() {
        await this.goShowMenu(1 /* ShowCases.onManualCall */, null, true);
    }
    /** Reference to popupMenu */
    $refPopup;
    #popupRefs;
    renderControl() {
        super.renderControl();
        const i = this.$refInput;
        i.setAttribute("role", "combobox");
        i.setAttribute("aria-haspopup", "listbox");
        i.setAttribute("aria-expanded", false);
    }
    gotChanges(propsChanged) {
        super.gotChanges(propsChanged);
        this._opts.readOnlyInput
            ? this.$refInput.removeAttribute("aria-autocomplete")
            : this.$refInput.setAttribute("aria-autocomplete", "list");
    }
    gotFormChanges(propsChanged) {
        super.gotFormChanges(propsChanged);
        this.$refInput.readOnly = this.$refInput.readOnly || this._opts.readOnlyInput;
        const isMenuEnabled = !this.$isDisabled && !this.$isReadOnly;
        if (!isMenuEnabled) {
            this.#popupRefs?.stopListen.call(this.#popupRefs); // remove all possible prev-eventListeners
            this.#popupRefs = undefined;
            this.removePopup();
        }
        else if (!this.#popupRefs) {
            const refs = popupListen({
                target: this,
                showCase: 4 /* PopupShowCases.onClick */ | 2 /* PopupShowCases.onFocus */,
                skipAlreadyFocused: true,
            }, (s, e) => {
                const sc = s === 4 /* PopupShowCases.onClick */ ? 4 /* ShowCases.onClick */ : 2 /* ShowCases.onFocus */;
                if (s === 0 /* PopupShowCases.always */) {
                    return this.$refPopup;
                }
                if (!(sc & this._opts.showCase)) {
                    return null;
                }
                return this.goShowMenu(sc, e);
            }, (s, e) => {
                if (s === 0 /* PopupHideCases.onManuallCall */) {
                    return true;
                }
                if (s !== 4 /* PopupHideCases.onPopupClick */) {
                    return this.goHideMenu(s === 2 /* PopupHideCases.onFocusOut */ || s === 3 /* PopupHideCases.onOutsideClick */
                        ? 3 /* HideCases.onFocusLost */
                        : 1 /* HideCases.onClick */, e);
                }
                return false;
            });
            this.#popupRefs = refs;
        }
    }
    /** Override to change show-behavior */
    canShowMenu(showCase, e) {
        if (this.$isReadOnly) {
            return false;
        }
        if (this._opts.showCase & 16 /* ShowCases.onClickInput */) {
            return true;
        }
        if (showCase === 4 /* ShowCases.onClick */ &&
            e.target instanceof HTMLInputElement &&
            !e.target.readOnly &&
            e.target.value !== "") {
            return false; // if input readonly > dropdown behavior otherwise allow to work with input instead of opening window
        }
        return true;
    }
    async goShowMenu(showCase, e, isNeedWait) {
        if (this.#isOpen) {
            return this.$refPopup;
        }
        if (!this.canShowMenu(showCase, e)) {
            return null;
        }
        this.#isOpen = true;
        // this.$hideError(); // it resolves overflow menu vs error
        if (!this.$refPopup) {
            const p = document.createElement("wup-popup");
            this.$refPopup = p;
            p.$options.showCase = 0 /* PopupShowCases.always */;
            p.$options.target = this;
            p.$options.offsetFitElement = [1, 1];
            p.$options.minWidthByTarget = true;
            p.$options.placement = [
                WUPPopupElement.$placements.$bottom.$start,
                WUPPopupElement.$placements.$bottom.$end,
                WUPPopupElement.$placements.$top.$start,
                WUPPopupElement.$placements.$top.$end,
                WUPPopupElement.$placements.$bottom.$start.$resizeHeight,
                WUPPopupElement.$placements.$bottom.$end.$resizeHeight,
                WUPPopupElement.$placements.$top.$start.$resizeHeight,
                WUPPopupElement.$placements.$top.$end.$resizeHeight,
            ];
            p.setAttribute("menu", "");
            p.$options.animation = 1 /* Animations.drawer */;
            const menuId = this.#ctr.$uniqueId;
            const i = this.$refInput;
            i.setAttribute("aria-owns", menuId);
            i.setAttribute("aria-controls", menuId);
            const wasFcs = this.$isFocused;
            await this.renderMenu(p, menuId);
            const fcs = this.$isFocused;
            if (!fcs && wasFcs) {
                this.#isOpen = false;
                this.removePopup(); // fix case when user waited for loading and moved focus to another
                return null;
            }
            this.appendChild(p); // WARN: it will show onInit
            this.$refPopup.addEventListener("$willShow", (ev) => ev.preventDefault(), { once: true }); // otherwise popup shows by init and impossible to wait for result (only via event)
            // eslint-disable-next-line no-promise-executor-return
            await new Promise((res) => setTimeout(res)); // wait for appending to body so size is defined and possible to scroll
        }
        if (!this.#isOpen) {
            return null; // possible when user calls show & hide sync
        }
        const r = this.$refPopup.$show().finally(() => this.fireEvent("$showMenu", { cancelable: false }));
        this.setAttribute("opened", ""); // possible when user calls show & hide sync
        this.$refInput.setAttribute("aria-expanded", true);
        this.#popupRefs.show(0 /* PopupShowCases.always */); // call for ref-listener to apply events properly
        isNeedWait && (await r); // WARN: it's important don't wait for animation to assign onShow events fast
        return this.$refPopup;
    }
    /** Override to change hide-behavior */
    canHideMenu(hideCase, e) {
        if (this._opts.showCase & 16 /* ShowCases.onClickInput */) {
            return true;
        }
        if (hideCase === 1 /* HideCases.onClick */ &&
            e.target instanceof HTMLInputElement &&
            !e.target.readOnly &&
            e.target.value !== "") {
            return false; // if input readonly > dropdown behavior otherwise allow to work with input instead of opening window
        }
        return true;
    }
    _isHidding;
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    async goHideMenu(hideCase, e) {
        if (!this.$refPopup || this._isHidding) {
            return false;
        }
        const wasOpen = this.#isOpen;
        if (!this.canHideMenu(hideCase, e)) {
            return false;
        }
        this.#isOpen = false;
        /* istanbul ignore else */
        if (wasOpen) {
            this._isHidding = true;
            this.#popupRefs.hide(0 /* PopupHideCases.onManuallCall */); // call for ref-listener to apply events properly
            await this.$refPopup.$hide();
            delete this._isHidding;
            if (this.#isOpen) {
                return false; // possible when popup opened again during the animation
            }
            // remove popup only by focusOut to optimize resources
            if (hideCase === 3 /* HideCases.onFocusLost */) {
                this.removePopup();
            }
        }
        this.removeAttribute("opened");
        this.$refInput.setAttribute("aria-expanded", false);
        this.focusMenuItem(null);
        this.selectMenuItem(null);
        setTimeout(() => this.fireEvent("$hideMenu", { cancelable: false }));
        return true;
    }
    _focusedMenuItem;
    /** Focus/resetFocus for item (via aria-activedescendant) */
    focusMenuItem(next) {
        this._focusedMenuItem?.removeAttribute("focused");
        if (next) {
            next.id = next.id || this.#ctr.$uniqueId;
            next.setAttribute("focused", "");
            this.$refInput.setAttribute("aria-activedescendant", next.id);
            const ifneed = next.scrollIntoViewIfNeeded;
            ifneed ? ifneed.call(next, false) : next.scrollIntoView();
        }
        else {
            this.$refInput.removeAttribute("aria-activedescendant");
        }
        this._focusedMenuItem = next;
    }
    _selectedMenuItem;
    /** Select item (set aria-selected and scroll to) */
    selectMenuItem(next) {
        this._selectedMenuItem?.setAttribute("aria-selected", false);
        if (next) {
            next.setAttribute("aria-selected", true);
            const ifneed = next.scrollIntoViewIfNeeded;
            ifneed ? ifneed.call(next, false) : next.scrollIntoView();
        }
        this._selectedMenuItem = next;
    }
    async gotKeyDown(e) {
        // don't allow to process Esc-key when menu is opened
        const isEscPrevent = this.#isOpen && e.key === "Escape";
        !isEscPrevent && super.gotKeyDown(e);
        if (e.altKey || e.shiftKey || e.ctrlKey) {
            return;
        }
        if (this._opts.showCase & 64 /* ShowCases.onPressArrowKey */) {
            if (e.key === "ArrowDown" || e.key === "ArrowUp") {
                e.preventDefault(); // to prevent parent-scroll
                if (!this.#isOpen) {
                    await this.goShowMenu(64 /* ShowCases.onPressArrowKey */, null); // , true);
                    return;
                }
            }
        }
        if (!this.#isOpen) {
            return;
        }
        switch (e.key) {
            case "Escape":
                e.preventDefault();
                this.resetInputValue();
                await this.goHideMenu(4 /* HideCases.OnPressEsc */);
                break;
            case "Enter":
                // case " ": user can type space; we should not use this as click
                e.preventDefault();
                {
                    const el = this._focusedMenuItem;
                    if (el && !el.hasAttribute("disabled")) {
                        const isHandled = !el.dispatchEvent(new MouseEvent("click", { cancelable: true, bubbles: true }));
                        if (isHandled) {
                            return; // skip hidding menu if itemClick isHandled
                        }
                    }
                    await this.goHideMenu(5 /* HideCases.OnPressEnter */);
                    this.resetInputValue();
                }
                break;
            default:
                break;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    gotInput(e, allowSuper = false) {
        // gotInput possible on browser-autofill so we need filter check if isFocused
        !this.$isOpen && this._opts.showCase & 32 /* ShowCases.onInput */ && this.$isFocused && this.goShowMenu(32 /* ShowCases.onInput */);
        allowSuper && super.gotInput(e);
    }
    gotFocusLost() {
        !this.#isOpen && !this._isHidding && this.removePopup(); // otherwise it's removed by hidingMenu
        this.resetInputValue(); // to update/rollback input according to result
        super.gotFocusLost();
    }
    /** Called when user selected new value from menu and need to hide menu */
    selectValue(v) {
        this.setValue(v);
        setTimeout(() => this.goHideMenu(2 /* HideCases.onSelect */)); // without timeout it handles click by listener and opens again
    }
    /* Reset input to currentValue; called on focusOut, press Escape, Enter */
    resetInputValue() {
        this.setInputValue(this.$value);
    }
    setInputValue(v) {
        const p = this.valueToInput(v);
        if (p instanceof Promise) {
            // todo try to avoid promise here and use super.setInputValue instead
            p.then((s) => super.setInputValue(s));
        }
        else {
            super.setInputValue(p);
        }
    }
    /** Called when popup must be removed (by focus out OR if control removed) */
    removePopup() {
        this.$refPopup?.remove();
        this.$refPopup = undefined;
        delete this._focusedMenuItem;
        if (this.#isOpen) {
            this.#isOpen = false;
            this.removeAttribute("opened");
            this.$refInput.setAttribute("aria-expanded", false);
            this.$refInput.removeAttribute("aria-activedescendant");
        }
    }
    gotRemoved() {
        this.removePopup();
        // remove resources for case when control can be appended again
        this.#popupRefs?.stopListen.call(this);
        this.#popupRefs = undefined;
        super.gotRemoved();
    }
}
// WARN about chaining
/*
 $showMenu().then(()=> console.warn('done'))
 isOpen: true >>> create popup,
 isOpening: true
 ...fetch menu-items (takes time)... >>> render popup with items
 ...animation (takes time)...
 isOpening: false
 >>> console.warn('done')
 show-event
 */
/*
 $hideMenu().then(()=> console.warn('done'))
 isOpen: false
 isHidding: true
 ...animation...
 isHidding: false >>> remove popup if focus out
 >>> console.warn('done')
 hide-event
 */
