import WUPPopupElement from "../popup/popupElement";
import WUPBaseComboControl from "./baseCombo";
import { ValidateFromCases } from "./baseControl";
declare const tagName = "wup-date";
declare global {
    namespace WUP.Date {
        interface EventMap extends WUP.BaseCombo.EventMap {
        }
        interface ValidityMap extends WUP.BaseCombo.ValidityMap, Pick<WUP.Text.ValidityMap, "_mask" | "_parse"> {
            /** Enabled if option [min] is pointed; If $value < pointed shows message 'Min date is {x}` */
            min: Date;
            /** Enabled if option [min] is pointed; if $value > pointed shows message 'Max date is {x}` */
            max: Date;
            /** Enabled if option [exclude] is pointed; If invalid shows "This date is disabled" */
            exclude: Date[];
        }
        interface Defaults<T = Date, VM = ValidityMap> extends WUP.Calendar.Defaults<T, VM>, WUP.BaseCombo.Defaults<T, VM> {
            /** String representation of displayed date (enables mask, - to disable mask set $options.mask="");
             * @defaultValue localeInfo.date
             * @tutorial Troubleshooting
             * * with changing $options.format need to change/reset mask/maskholder also */
            format?: string;
        }
        interface Options<T = Date, VM = ValidityMap> extends WUP.Calendar.Options<T, VM>, WUP.BaseCombo.Options<T, VM>, Defaults<T, VM> {
            format: string;
            firstWeekDay: number;
        }
        interface Attributes extends WUP.Calendar.Attributes, Pick<Defaults, "format"> {
        }
        interface JSXProps<C = WUPDateControl> extends WUP.BaseCombo.JSXProps<C>, WUP.Calendar.JSXProps<C>, Attributes {
            initValue?: string;
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPDateControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Date.JSXProps;
        }
    }
}
/** Form-control with date picker
 * @tutorial Troubleshooting
 * * $options.format related only to displayed text, to work with other date-options like min/max use strict format 'YYYY-MM-DD'
 * @example
  const el = document.createElement("wup-date");
  el.$options.name = "dateOfBirthday";
  el.$initValue = "1990-10-24";
  el.$options.validations = { required: true };
  el.$options.format = "yyyy-MM-dd";
  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-date name="dateOfBirhday" utc initvalue="1990-10-24" min="1930-01-01" max="2010-01-01"/>
  </wup-form>;
 */
export default class WUPDateControl<ValueType extends Date = Date, EventMap extends WUP.Date.EventMap = WUP.Date.EventMap> extends WUPBaseComboControl<ValueType, EventMap> {
    #private;
    static get $styleRoot(): string;
    static get $style(): string;
    static get observedOptions(): Array<string>;
    static get observedAttributes(): Array<string>;
    static $defaults: WUP.Date.Defaults;
    $options: WUP.Date.Options;
    protected _opts: WUP.Date.Options<Date, WUP.Date.ValidityMap>;
    /** Parse string to Date
     * @see WUPCalendarControl.$parse */
    parse(text: string): ValueType | undefined;
    /** Called to parse input text to value (related to locale or pointed format)
     *  @tutorial Troubleshooting
     * * for "yyyy-mm-dd" the correct format is "yyyy-MM-dd" */
    parseInput(text: string): ValueType | undefined;
    canParseInput(_text: string): boolean;
    protected gotChanges(propsChanged: Array<keyof WUP.Date.Options> | null): void;
    protected get validations(): WUP.Text.Options["validations"];
    protected goValidate(fromCase: ValidateFromCases, silent?: boolean): string | false;
    protected renderMenu(popup: WUPPopupElement, menuId: string): Promise<HTMLElement>;
    protected setValue(v: ValueType | undefined, canValidate?: boolean, skipInput?: boolean): boolean | null;
    protected valueToInput(v: ValueType | undefined): Promise<string> | string;
    protected resetInputValue(): void;
    protected gotInput(e: WUP.Text.GotInputEvent): void;
    protected gotKeyDown(e: KeyboardEvent): Promise<void>;
    protected focusMenuItem(next: HTMLElement | null): void;
}
export {};
