import WUPTextControl from "./text";
declare const tagName = "wup-num";
declare global {
    namespace WUP.Number {
        interface Format {
            /** Decimal separator; for number 123.4 it's dot
             * localeInfoInfo @see localeInfo.sepDecimal */
            sepDecimal?: string;
            /** Thouthands separator; for number 1,234.5 it's comma
             *  localeInfoInfo @see localeInfo.sep1000 */
            sep1000?: string;
            /** Maximum displayed fraction digits; for 123.45 it's 2
             * @defaultValue 0 */
            maxDecimal?: number;
            /** Minimun displayed fraction digits; if pointed 2 then 123.4 goes to 123.40
             * @defaultValue 0 */
            minDecimal?: number;
        }
        interface EventMap extends WUP.BaseControl.EventMap {
        }
        interface ValidityMap extends WUP.BaseControl.ValidityMap, Pick<WUP.Text.ValidityMap, "_mask" | "min" | "max"> {
            /** If $value < pointed shows message 'Min value {x}` */
            min: number;
            /** If $value < pointed shows message 'Max value {x}` */
            max: number;
        }
        interface Defaults<T = number, VM extends ValidityMap = ValidityMap> extends WUP.Text.Defaults<T, VM> {
        }
        interface Options<T = number, VM extends ValidityMap = ValidityMap> extends WUP.Text.Options<T, VM>, Defaults<T, VM> {
            /** String representation of displayed value */
            format?: Format;
        }
        interface Attributes extends WUP.Text.Attributes {
        }
        interface JSXProps<C = WUPNumberControl> extends WUP.Text.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPNumberControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Number.JSXProps;
        }
    }
}
/** Form-control with number-input
 * @example
  const el = document.createElement("wup-num");
  el.$options.name = "number";
  el.$options.validations = {
    min: 1,
    max: 1000,
  };

  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-num name="number" validations="myValidations"/>
  </wup-form>;
 * @see WUPTextControl
 */
export default class WUPNumberControl<ValueType = number, EventMap extends WUP.Number.EventMap = WUP.Number.EventMap> extends WUPTextControl<ValueType, EventMap> {
    #private;
    static get observedOptions(): Array<string>;
    /** Default options - applied to every element. Change it to configure default behavior */
    static $defaults: WUP.Number.Defaults;
    $options: WUP.Number.Options;
    protected _opts: WUP.Number.Options<number, WUP.Number.ValidityMap>;
    /** Returns $options.format joined with defaults */
    get $format(): Required<WUP.Number.Format>;
    /** Called when need to format value */
    protected valueToInput(v: ValueType | undefined): string;
    parse(text: string): ValueType | undefined;
    canParseInput(text: string): boolean;
    parseInput(text: string): ValueType | undefined;
    protected canHandleUndo(): boolean;
    protected setInputValue(v: ValueType | undefined): void;
    protected renderControl(): void;
    protected gotChanges(propsChanged: Array<keyof WUP.Number.Options> | null): void;
    protected gotBeforeInput(e: WUP.Text.GotInputEvent): void;
    private _canShowDeclined?;
    protected gotInput(e: WUP.Text.GotInputEvent): void;
    protected gotFocus(): Array<() => void>;
    _isAltDown?: true;
    _isShiftDown?: true;
    _isCtrlDown?: true;
    protected gotKeyDown(e: KeyboardEvent): void;
    /** Called when user tries to increment/decrement value (via ArrowKeys/Mouse/Swipe) */
    protected gotIncrement(dval: number): void;
}
export {};
