import WUPTextControl from "./text";
declare const tagName = "wup-pwd";
declare global {
    namespace WUP.Password {
        interface EventMap extends WUP.Text.EventMap {
        }
        interface ValidityMap extends WUP.Text.ValidityMap {
            /** If count of numbers < pointed shows message 'Must contain at least {x} numbers' */
            minNumber: number;
            /** If count of uppercase letters < pointed shows message 'Must contain at least {x} upper case' */
            minUpper: number;
            /** If count of lowercase letters < pointed shows message 'Must contain at least {x} lower case' */
            minLower: number;
            /** If count of pointed chars < pointed shows message 'Must contain at least {x} special characters' */
            special: {
                min: number;
                chars: string;
            };
            /** If $value != with previous siblint wup-pwd.$value shows message 'Passwords must be equal' */
            confirm: boolean;
        }
        interface Defaults<T = string, VM extends ValidityMap = ValidityMap> extends WUP.Text.Defaults<T, VM> {
        }
        interface Options<T = string, VM extends ValidityMap = ValidityMap> extends WUP.Text.Options<T, VM>, Defaults<T, VM> {
            /** Reversed-style for button-eye
             * @defaultValue false */
            reverse?: boolean;
        }
        interface Attributes extends WUP.Text.Attributes {
            /** Reversed-style for button-eye */
            reverse?: boolean | "";
        }
        interface JSXProps<C = WUPPasswordControl> extends WUP.Text.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPPasswordControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Password.JSXProps<WUPPasswordControl>;
        }
    }
}
/** Form-control with text-input
 * @example
  const el = document.createElement("wup-pwd");
  el.$options.name = "password";
  el.$options.validations = {
    required: true,
    min: 8,
    minNumber: 1,
    minUpper: 1,
    minLower: 1,
    special: { min: 1, chars: "#!-_?,.@:;'" }
  };
  el.$options.validationShowAll = true;

  const elConfirm = document.createElement("wup-pwd");
  elConfirm.$options.name = "passwordConfirm";
  elConfirm.$options.validations = { confirm: true };

  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-pwd name="password" validations="myValidations"/>
    <wup-pwd name="passwordConfirm" validations="myValidations2"/>
  </wup-form>;
 * @tutorial innerHTML @example
 * <label>
 *   <span> // extra span requires to use with icons via label:before, label:after without adjustments
 *      <input/>
 *      <strong>{$options.label}</strong>
 *   </span>
 *   <button clear/>
 * </label>
 */
export default class WUPPasswordControl<ValueType extends string = string, EventMap extends WUP.Password.EventMap = WUP.Password.EventMap> extends WUPTextControl<ValueType, EventMap> {
    #private;
    static get observedOptions(): Array<string>;
    /** Text announced by screen-readers when input cleared; @defaultValue `input cleared` */
    static $ariaDescription: string;
    static get $styleRoot(): string;
    static get $style(): string;
    /** Default options - applied to every element. Change it to configure default behavior */
    static $defaults: WUP.Password.Defaults;
    $options: WUP.Password.Options<string>;
    protected _opts: WUP.Password.Options<string, WUP.Password.ValidityMap>;
    $refBtnEye: HTMLButtonElement;
    protected renderControl(): void;
    /** Create & append element to control */
    protected renderBtnEye(): void;
    protected toggleVisibility(): void;
    protected gotChanges(propsChanged: Array<keyof WUP.Password.Options> | null): void;
    protected gotKeyDown(e: KeyboardEvent): void;
}
export {};
