import WUPBaseControl from "./baseControl";
declare const tagName = "wup-radio";
declare global {
    namespace WUP.Radio {
        interface EventMap extends WUP.BaseControl.EventMap {
        }
        interface ValidityMap extends WUP.BaseControl.ValidityMap {
        }
        interface Defaults<T = any, VM = ValidityMap> extends WUP.BaseControl.Defaults<T, VM> {
        }
        interface Options<T = any, VM = ValidityMap> extends WUP.BaseControl.Options<T, VM>, Defaults<T, VM> {
            /** Items showed as radio-buttons */
            items: WUP.Select.MenuItems<T> | (() => WUP.Select.MenuItems<T>);
            /** Reversed-style (radio+label for true vs label+radio)
             * @defaultValue false */
            reverse?: boolean;
        }
        interface Attributes extends WUP.BaseControl.Attributes {
            /** Items showed as radio-buttons. Point global obj-key with items (set `window.inputRadio.items` for `window.inputRadio.items = [{value: 1, text: 'Item 1'}]` ) */
            items?: string;
            /** Reversed-style (radio+label vs label+radio) */
            reverse?: boolean | "";
        }
        interface JSXProps<C = WUPRadioControl> extends WUP.BaseControl.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPRadioControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Radio.JSXProps<WUPRadioControl>;
        }
    }
}
interface ExtInputElement extends HTMLInputElement {
    _value: any;
}
/** Form-control with radio buttons
 * @example
  const el = document.createElement("wup-radio");
  el.$options.name = "gender";
  el.$options.items = [
    { value: 1, text: "Male" },
    { value: 2, text: "Female" },
    { value: 3, text: "Other/Skip" },
  ];
  el.$initValue = 3;
  el.$options.validations = { required: true };
  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-radio name="gender" initvalue="3" validations="myValidations" items="myRadioItems"/>
  </wup-form>;
 * @tutorial innerHTML @example
 * <fieldset>
 *   <legend><strong>{$options.label}</strong></legend>
 *   <label> <input/> <span/> </label>
 *   <label> <input/> <span/> </label>
 *   // etc.
 * </fieldset>
 */
export default class WUPRadioControl<ValueType = any, EventMap extends WUP.Radio.EventMap = WUP.Radio.EventMap> extends WUPBaseControl<ValueType, EventMap> {
    #private;
    /** Custom text that announced by screen-readers. Redefine it to use with another language */
    static $ariaReadonly: string;
    static get $styleRoot(): string;
    static get $style(): string;
    static get observedOptions(): Array<string>;
    static get observedAttributes(): Array<string>;
    static $defaults: WUP.Radio.Defaults;
    $options: WUP.Radio.Options;
    protected _opts: WUP.Radio.Options<any, WUP.Radio.ValidityMap>;
    /** Called when need to parse attr [initValue] */
    parse(attrValue: string): ValueType | undefined;
    protected gotReady(): void;
    /** Called when user touches inputs */
    protected gotInput(e: Event & {
        target: ExtInputElement;
    }): void;
    protected getItems(): WUP.Select.MenuItems<ValueType>;
    $refFieldset: HTMLFieldSetElement;
    $refItems: ExtInputElement[];
    protected renderControl(): void;
    protected renderItems(parent: HTMLFieldSetElement): void;
    /** Called when need to update check-state of inputs */
    protected checkInput(v: ValueType | undefined): void;
    protected setValue(v: ValueType, canValidate?: boolean): boolean | null;
    protected gotChanges(propsChanged: Array<keyof WUP.Radio.Options> | null): void;
    gotFormChanges(propsChanged: Array<keyof WUP.Form.Options> | null): void;
    protected gotOptionsChanged(e: WUP.Base.OptionEvent): void;
    focus(): boolean;
    protected goShowError(err: string, target: HTMLElement): void;
}
export {};
