import WUPPopupElement from "../popup/popupElement";
import WUPSpinElement from "../spinElement";
import WUPBaseComboControl, { ShowCases } from "./baseCombo";
declare const tagName = "wup-select";
declare global {
    namespace WUP.Select {
        interface MenuItemText<T> {
            text: string;
            value: T;
        }
        /** Use li.innerHTML to render value & return string required for input; */
        interface MenuItemFn<T> {
            text: (value: T, el: HTMLElement, i: number) => string;
            value: T;
        }
        type MenuItem<T> = MenuItemText<T> | MenuItemFn<T>;
        type MenuItems<T> = MenuItemText<T>[] | MenuItemFn<T>[];
        interface EventMap extends WUP.BaseCombo.EventMap {
        }
        interface ValidityMap extends WUP.BaseCombo.ValidityMap {
        }
        interface Defaults<T = any, VM = ValidityMap> extends WUP.BaseCombo.Defaults<T, VM> {
            /** Case when menu-popup to show
             * @defaultValue onPressArrowKey | onClick | onFocus | onInput */
            showCase: ShowCases;
        }
        interface Options<T = any, VM = ValidityMap> extends WUP.BaseCombo.Options<T, VM>, Defaults<T, VM> {
            /** Items showed in dropdown-menu. Provide promise/api-call to show pending status when control retrieves data! */
            items: MenuItems<T> | (() => MenuItems<T> | Promise<MenuItems<T>>);
            /** Set true to make input not editable but allow to user select items via popup-menu (ordinary dropdown mode) */
            readOnlyInput?: boolean;
            /** Allow user to create new value if value not found in items */
            allowNewValue?: boolean;
        }
        interface Attributes extends WUP.BaseCombo.Attributes {
        }
        interface JSXProps<C = WUPSelectControl> extends WUP.BaseCombo.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPSelectControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Select.JSXProps;
        }
    }
}
/** Form-control with dropdown/combobox behavior
 * @example
  const el = document.createElement("wup-select");
  el.$options.name = "gender";
  el.$options.items = [
    { value: 1, text: "Male" },
    { value: 2, text: "Female" },
    { value: 3, text: "Other/Skip" },
  ];
  el.$initValue = 3;
  el.$options.validations = { required: true };
  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-select name="gender" initvalue="3" validations="myValidations" items="myDropdownItems" />
  </wup-form>;
 * @tutorial innerHTML @example
 * <label>
 *   <span> // extra span requires to use with icons via label:before, label:after without adjustments
 *      <input/>
 *      <strong>{$options.label}</strong>
 *   </span>
 *   <button clear/>
 *   <wup-popup menu>
 *      <ul>
 *          <li>Item 1</li>
 *          <li>Item 2</li>
 *          // etc/
 *      </ul>
 *   </wup-popup>
 * </label>
 */
export default class WUPSelectControl<ValueType = any, ItemType = ValueType, EventMap extends WUP.Select.EventMap = WUP.Select.EventMap> extends WUPBaseComboControl<ValueType, EventMap> {
    #private;
    static get observedOptions(): Array<string>;
    static get $styleRoot(): string;
    static get $style(): string;
    /** Text for listbox when no items are displayed */
    static $textNoItems: string | undefined;
    /** Text for aria-label of <ul> element */
    static $ariaLabelItems: string;
    /** Function to filter menuItems based on inputValue; all values are in lowerCase */
    static $filterMenuItem(menuItemText: string, inputValue: string, inputRawValue: string): boolean;
    static $defaults: WUP.Select.Defaults;
    $options: WUP.Select.Options;
    protected _opts: WUP.Select.Options<any, WUP.Select.ValidityMap>;
    /** Returns whether control in pending state or not (shows spinner) */
    get $isPending(): boolean;
    /** Called when need to parse attr [initValue] */
    parse(attrValue: string): ValueType | undefined;
    /** Called on every spin-render */
    renderSpin(): WUPSpinElement;
    protected gotOptionsChanged(e: WUP.Base.OptionEvent): Promise<void>;
    gotFormChanges(propsChanged: Array<keyof WUP.Form.Options> | null): void;
    /** Change pending state */
    protected changePending(v: boolean): void;
    /** All items of current menu */
    _menuItems?: {
        all: Array<HTMLLIElement & {
            _text: string;
        }>;
        /** Index of items filtered by input */
        filtered?: Array<number>;
        /** Index (in 'filtered' otherwise in 'all' array) of item that has virtual-focus; */
        focused: number;
    };
    /** Items resolved from options */
    _cachedItems?: WUP.Select.MenuItems<ValueType>;
    /** Called when NoItems need to show */
    protected renderMenuNoItems(popup: WUPPopupElement, isReset: boolean): void;
    protected renderMenu(popup: WUPPopupElement, menuId: string): Promise<HTMLElement>;
    /** Method retrieves items from options and shows spinner if required */
    protected getItems(): Promise<WUP.Select.MenuItems<ValueType>>;
    protected valueToText(v: ItemType, items: WUP.Select.MenuItems<ItemType>): string;
    protected valueToInput(v: ValueType | undefined): Promise<string> | string;
    /** It's called with option allowNewValue to find value related to text */
    protected findValueByText(txt: string): ValueType | undefined;
    /** Create menuItems as array of HTMLLiElement with option _text required to filtering by input (otherwise content can be html-structure) */
    protected renderMenuItems(ul: HTMLUListElement): Promise<Array<HTMLLIElement & {
        _text: string;
    }>>;
    /** Called when need to setValue & close base on clicked item */
    protected gotMenuItemClick(_e: MouseEvent, item: HTMLLIElement & {
        _text: string;
    }): void;
    protected goShowMenu(showCase: ShowCases, e?: MouseEvent | FocusEvent | null, isNeedWait?: boolean): Promise<WUPPopupElement | null>;
    /** Select menu item if value !== undefined and menu is opened */
    protected selectMenuItemByValue(v: ValueType | undefined): void;
    protected focusMenuItem(next: HTMLElement | null): void;
    /** Focus item by index or reset is index is null (via aria-activedescendant).
     *  If menuItems is filtered by input-text than index must point on filtered array */
    protected focusMenuItemByIndex(index: number): void;
    protected gotKeyDown(e: KeyboardEvent): Promise<void>;
    /** For case when need to menu is opened but items are not rendered yet */
    protected _needFilter?: () => void;
    protected gotInput(e: WUP.Text.GotInputEvent): void;
    protected gotFocusLost(): void;
    protected setValue(v: ValueType | undefined, canValidate?: boolean, skipInput?: boolean): boolean | null;
    protected removePopup(): void;
}
export {};
