import WUPTextControl from "./text";
declare const tagName = "wup-textarea";
declare global {
    namespace WUP.Textarea {
        interface EventMap extends WUP.Text.EventMap {
        }
        interface ValidityMap extends WUP.Text.ValidityMap {
        }
        interface Defaults<T = string, VM = ValidityMap> extends WUP.Text.Defaults<T, VM> {
        }
        interface Options<T = string, VM = ValidityMap> extends Omit<WUP.Text.Options<T, VM>, "mask" | "maskholder" | "prefix" | "postfix">, Defaults<T, VM> {
        }
        interface Attributes extends WUP.Text.Attributes {
        }
        interface JSXProps<C = WUPTextareaControl> extends WUP.Text.JSXProps<C>, Attributes {
        }
    }
    interface HTMLElementTagNameMap {
        [tagName]: WUPTextareaControl;
    }
    namespace JSX {
        interface IntrinsicElements {
            [tagName]: WUP.Textarea.JSXProps;
        }
    }
}
/** Form-control with text-input
 * @example
  const el = document.createElement("wup-textarea");
  el.$options.name = "textarea";
  el.$options.validations = {
    required: true,
    max: 250
  };

  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-textarea name="textarea" validations="myValidations"/>
  </wup-form>;
 * @tutorial innerHTML @example
 * <label>
 *   <span> // extra span requires to use with icons via label:before, label:after without adjustments
 *      <span contenteditable="true" />
 *      <strong>{$options.label}</strong>
 *   </span>
 *   <button clear/>
 * </label>
 */
export default class WUPTextareaControl<ValueType = string, EventMap extends WUP.Textarea.EventMap = WUP.Textarea.EventMap> extends WUPTextControl<ValueType, EventMap> {
    #private;
    static get $style(): string;
    /** Default options - applied to every element. Change it to configure default behavior */
    static $defaults: WUP.Textarea.Defaults;
    $options: WUP.Textarea.Options;
    protected _opts: WUP.Textarea.Options<string, WUP.Textarea.ValidityMap>;
    $refInput: HTMLInputElement & HTMLSpanElement;
    constructor();
    /** Fix for contenteditable (it doesn't contain selectionStart & selectionEnd props) */
    private get selection();
    private set selection(value);
    protected renderControl(): void;
    parseInput(text: string): ValueType | undefined;
    protected gotChanges(propsChanged: Array<keyof WUP.Textarea.Options> | null): void;
    protected renderPrefix(): void;
    protected renderPostfix(): void;
    protected gotKeyDown(e: KeyboardEvent & {
        submitPrevented?: boolean;
    }): void;
    protected gotFocusLost(): void;
}
export {};
