declare global {
    namespace WUP {
        interface PromiseCancel<T> extends Promise<T> {
            /** Call it when you need to stop promise immediately;
             *  @param isFinish finish promise immediately @defaultValue true
             *  @tutorial
             * * isFinish:true - Promise is resolved
             * * isFinish:false - Promise is rejected
             * @returns promise resolved when animation stop process is done */
            stop: (isFinish?: boolean) => Promise<T>;
        }
    }
}
/** Animate (open/close) element as dropdown via scale and counter-scale for children
 * @param ms animation time
 * @returns Promise<isFinished> that resolved by animation end;
 * Rules
 * * To define direction set attribute to element position="top" or position="bottom"
 * * Before call it again on the same element don't forget to call stop(false) to cancel prev animation */
export declare function animateDropdown(el: HTMLElement, ms: number, isClose?: boolean): WUP.PromiseCancel<boolean>;
/** Animation function based on window.requestAnimationFrame
 * @param from animate from value
 * @param to animate to value
 * @param ms time of animation
 * @param callback fired on every window.requestAnimationFrame() is fired
 * @param force set `true` to ignore window.matchMedia("(prefers-reduced-motion)")
 * @returns promise with extra functions; resolves `true` when animation is finished or resolves `false` when animation is canceled */
export declare function animate(from: number, to: number, ms: number, callback: (v: number, ms: number, isLast: boolean, timeStamp: number) => void, force?: boolean): WUP.PromiseCancel<boolean>;
