import { findScrollParentAll } from "./findScrollParent";
import { getBoundingInternalRect } from "./styleHelpers";
/** Check if element is visible in scrollable parents */
export default function isIntoView(el, options) {
    const scrollParents = options?.scrollParents || findScrollParentAll(el) || [document.body];
    let child = options?.elRect ?? el.getBoundingClientRect();
    if (options?.offset) {
        child = {
            top: child.top - options.offset[0],
            left: child.left - (options.offset[3] ?? options.offset[1]),
            right: child.right + options.offset[1],
            bottom: child.bottom + (options.offset[2] ?? options.offset[0]),
        };
    }
    const vH = Math.max(document.documentElement.clientHeight, window.innerHeight);
    const vW = Math.max(document.documentElement.clientWidth, window.innerWidth);
    const r = {
        hiddenX: false,
        hiddenY: false,
        partialHiddenX: false,
        partialHiddenY: false,
        visible: true,
        hidden: false,
        partialVisible: false,
    };
    for (let i = 0; i < scrollParents.length; ++i) {
        let isVisY = false;
        let isVisX = false;
        const elRect = scrollParents[i].getBoundingClientRect();
        const p = (() => {
            if (!p._saved) {
                const s = getComputedStyle(scrollParents[i]);
                isVisY = s.overflowY === "visible";
                isVisX = s.overflowX === "visible";
                p._saved = getBoundingInternalRect(scrollParents[i], { computedStyle: s, elRect });
            }
            return p._saved; // implemented to cache huge getComputedStyle/getBoundingInternalRect
        });
        const isHiddenY = // checking if visible in viewPort also
         ((child.top > vH || child.bottom < 0) && document.documentElement) ||
            ((p().top >= child.bottom || p().bottom <= child.top) && !isVisY && scrollParents[i]);
        const isHiddenX = ((child.left > vW || child.right < 0) && document.documentElement) ||
            ((p().left >= child.right || p().right <= child.left) && !isVisX && scrollParents[i]);
        /* istanbul ignore else */
        if (isHiddenY) {
            r.hiddenY = isHiddenY;
        }
        else if (!r.hiddenY) {
            r.partialHiddenY =
                ((child.top < 0 || child.bottom > vH) && document.documentElement) ||
                    ((child.top < p().top || child.bottom > p().bottom) && !isVisY && scrollParents[i]);
        }
        /* istanbul ignore else */
        if (isHiddenX) {
            r.hiddenX = isHiddenX;
        }
        else if (!r.hiddenX) {
            r.partialHiddenX =
                ((child.left < 0 || child.right > vW) && document.documentElement) ||
                    ((child.left < p().left || child.right > p().right) && !isVisX && scrollParents[i]);
        }
        child = elRect;
    }
    r.hidden = r.hiddenX || r.hiddenY;
    r.visible = !r.hidden && !r.partialHiddenX && !r.partialHiddenY;
    r.partialVisible = !r.hidden && (r.partialHiddenX || r.partialHiddenY);
    return r;
}
