import { HideCases, ShowCases } from "./popupElement.types";
/** Listen for target according to showCase. If target removed call stopListen + don't forget to remove popup yourself;
 * @tutorial Troubleshooting/rules:
 * * To allow user hide/show popup before previous action is ended provide for onShow and onHide-
 *   callbacks results as fast as it's possible without waiting for animation */
declare function popupListen(options: {
    target: HTMLElement;
    showCase?: ShowCases;
    hoverShowTimeout?: number;
    hoverHideTimeout?: number;
    focusDebounceMs?: number;
    /** By default when listener is applied on focused element it provides immediate onShow event. You can disable this */
    skipAlreadyFocused?: boolean;
}, 
/** If succesfull callback must return HTMLElement */
onShow: (showCase: ShowCases, ev: MouseEvent | FocusEvent | null) => HTMLElement | null | Promise<HTMLElement | null>, onHide: (hideCase: HideCases, ev: MouseEvent | FocusEvent | null) => boolean | Promise<boolean>): {
    /** Fire it when target is removed manually (to remove all added related eventListeners) */
    stopListen: () => void;
    /** Fire it before hiding (to ability to show in the feature by listened events) when you need to hide outside listener (manually) */
    hide: (hideCase: HideCases) => Promise<void>;
    /** Fire it before showing (to ability to hide in the feature by listened events) when you need to show outside listener (manually) */
    show: (showCase: ShowCases) => Promise<void>;
};
declare namespace popupListen {
    var $defaults: {
        showCase: ShowCases;
        hoverShowTimeout: number;
        hoverHideTimeout: number;
    };
}
export default popupListen;
